@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import asyncio
 import contextlib
 import socket
 import ssl
@@ -9,17 +8,45 @@ from pathlib import Path
 
 import pytest
 import trustme
-from tornado import web
 
-from dummyserver.handlers import TestingApp
-from dummyserver.proxy import ProxyHandler
-from dummyserver.server import HAS_IPV6, run_loop_in_thread, run_tornado_app
-from dummyserver.testcase import HTTPSDummyServerTestCase
+import urllib3.http2
+import urllib3.http2.probe as http2_probe
+from dummyserver.app import hypercorn_app
+from dummyserver.asgi_proxy import ProxyApp
+from dummyserver.hypercornserver import run_hypercorn_in_thread
+from dummyserver.socketserver import HAS_IPV6
+from dummyserver.testcase import HTTPSHypercornDummyServerTestCase
 from urllib3.util import ssl_
 
 from .tz_stub import stub_timezone_ctx
 
 
+def pytest_addoption(parser: pytest.Parser) -> None:
+    parser.addoption(
+        "--integration",
+        action="store_true",
+        default=False,
+        help="run integration tests only",
+    )
+
+
+def pytest_collection_modifyitems(
+    config: pytest.Config, items: list[pytest.Item]
+) -> None:
+    integration_mode = bool(config.getoption("--integration"))
+    skip_integration = pytest.mark.skip(
+        reason="skipping, need --integration option to run"
+    )
+    skip_normal = pytest.mark.skip(
+        reason="skipping non integration tests in --integration mode"
+    )
+    for item in items:
+        if "integration" in item.keywords and not integration_mode:
+            item.add_marker(skip_integration)
+        elif integration_mode and "integration" not in item.keywords:
+            item.add_marker(skip_normal)
+
+
 class ServerConfig(typing.NamedTuple):
     scheme: str
     host: str
@@ -48,22 +75,13 @@ def _write_cert_to_dir(
 @contextlib.contextmanager
 def run_server_in_thread(
     scheme: str, host: str, tmpdir: Path, ca: trustme.CA, server_cert: trustme.LeafCert
-) -> typing.Generator[ServerConfig, None, None]:
+) -> typing.Generator[ServerConfig]:
     ca_cert_path = str(tmpdir / "ca.pem")
     ca.cert_pem.write_to_path(ca_cert_path)
     server_certs = _write_cert_to_dir(server_cert, tmpdir)
 
-    with run_loop_in_thread() as io_loop:
-
-        async def run_app() -> int:
-            app = web.Application([(r".*", TestingApp)])
-            server, port = run_tornado_app(app, server_certs, scheme, host)
-            return port
-
-        port = asyncio.run_coroutine_threadsafe(
-            run_app(), io_loop.asyncio_loop  # type: ignore[attr-defined]
-        ).result()
-        yield ServerConfig("https", host, port, ca_cert_path)
+    with run_hypercorn_in_thread(host, server_certs, hypercorn_app) as port:
+        yield ServerConfig(scheme, host, port, ca_cert_path)
 
 
 @contextlib.contextmanager
@@ -74,37 +92,29 @@ def run_server_and_proxy_in_thread(
     ca: trustme.CA,
     proxy_cert: trustme.LeafCert,
     server_cert: trustme.LeafCert,
-) -> typing.Generator[tuple[ServerConfig, ServerConfig], None, None]:
+) -> typing.Generator[tuple[ServerConfig, ServerConfig]]:
     ca_cert_path = str(tmpdir / "ca.pem")
     ca.cert_pem.write_to_path(ca_cert_path)
 
     server_certs = _write_cert_to_dir(server_cert, tmpdir)
     proxy_certs = _write_cert_to_dir(proxy_cert, tmpdir, "proxy")
 
-    with run_loop_in_thread() as io_loop:
+    with contextlib.ExitStack() as stack:
+        port = stack.enter_context(
+            run_hypercorn_in_thread("localhost", server_certs, hypercorn_app)
+        )
+        proxy_port = stack.enter_context(
+            run_hypercorn_in_thread(proxy_host, proxy_certs, ProxyApp())
+        )
 
-        async def run_app() -> tuple[ServerConfig, ServerConfig]:
-            app = web.Application([(r".*", TestingApp)])
-            server_app, port = run_tornado_app(app, server_certs, "https", "localhost")
-            server_config = ServerConfig("https", "localhost", port, ca_cert_path)
-
-            proxy = web.Application([(r".*", ProxyHandler)])
-            proxy_app, proxy_port = run_tornado_app(
-                proxy, proxy_certs, proxy_scheme, proxy_host
-            )
-            proxy_config = ServerConfig(
-                proxy_scheme, proxy_host, proxy_port, ca_cert_path
-            )
-            return proxy_config, server_config
-
-        proxy_config, server_config = asyncio.run_coroutine_threadsafe(
-            run_app(), io_loop.asyncio_loop  # type: ignore[attr-defined]
-        ).result()
-        yield (proxy_config, server_config)
+        yield (
+            ServerConfig(proxy_scheme, proxy_host, proxy_port, ca_cert_path),
+            ServerConfig("https", "localhost", port, ca_cert_path),
+        )
 
 
 @pytest.fixture(params=["localhost", "127.0.0.1", "::1"])
-def loopback_host(request: typing.Any) -> typing.Generator[str, None, None]:
+def loopback_host(request: typing.Any) -> typing.Generator[str]:
     host = request.param
     if host == "::1" and not HAS_IPV6:
         pytest.skip("Test requires IPv6 on loopback")
@@ -114,7 +124,7 @@ def loopback_host(request: typing.Any) -> typing.Generator[str, None, None]:
 @pytest.fixture()
 def san_server(
     loopback_host: str, tmp_path_factory: pytest.TempPathFactory
-) -> typing.Generator[ServerConfig, None, None]:
+) -> typing.Generator[ServerConfig]:
     tmpdir = tmp_path_factory.mktemp("certs")
     ca = trustme.CA()
 
@@ -127,7 +137,7 @@ def san_server(
 @pytest.fixture()
 def no_san_server(
     loopback_host: str, tmp_path_factory: pytest.TempPathFactory
-) -> typing.Generator[ServerConfig, None, None]:
+) -> typing.Generator[ServerConfig]:
     tmpdir = tmp_path_factory.mktemp("certs")
     ca = trustme.CA()
     server_cert = ca.issue_cert(common_name=loopback_host)
@@ -139,7 +149,7 @@ def no_san_server(
 @pytest.fixture()
 def no_san_server_with_different_commmon_name(
     tmp_path_factory: pytest.TempPathFactory,
-) -> typing.Generator[ServerConfig, None, None]:
+) -> typing.Generator[ServerConfig]:
     tmpdir = tmp_path_factory.mktemp("certs")
     ca = trustme.CA()
     server_cert = ca.issue_cert(common_name="example.com")
@@ -151,7 +161,7 @@ def no_san_server_with_different_commmon_name(
 @pytest.fixture
 def san_proxy_with_server(
     loopback_host: str, tmp_path_factory: pytest.TempPathFactory
-) -> typing.Generator[tuple[ServerConfig, ServerConfig], None, None]:
+) -> typing.Generator[tuple[ServerConfig, ServerConfig]]:
     tmpdir = tmp_path_factory.mktemp("certs")
     ca = trustme.CA()
     proxy_cert = ca.issue_cert(loopback_host)
@@ -166,7 +176,7 @@ def san_proxy_with_server(
 @pytest.fixture
 def no_san_proxy_with_server(
     tmp_path_factory: pytest.TempPathFactory,
-) -> typing.Generator[tuple[ServerConfig, ServerConfig], None, None]:
+) -> typing.Generator[tuple[ServerConfig, ServerConfig]]:
     tmpdir = tmp_path_factory.mktemp("certs")
     ca = trustme.CA()
     # only common name, no subject alternative names
@@ -182,7 +192,7 @@ def no_san_proxy_with_server(
 @pytest.fixture
 def no_localhost_san_server(
     tmp_path_factory: pytest.TempPathFactory,
-) -> typing.Generator[ServerConfig, None, None]:
+) -> typing.Generator[ServerConfig]:
     tmpdir = tmp_path_factory.mktemp("certs")
     ca = trustme.CA()
     # non localhost common name
@@ -195,7 +205,7 @@ def no_localhost_san_server(
 @pytest.fixture
 def ipv4_san_proxy_with_server(
     tmp_path_factory: pytest.TempPathFactory,
-) -> typing.Generator[tuple[ServerConfig, ServerConfig], None, None]:
+) -> typing.Generator[tuple[ServerConfig, ServerConfig]]:
     tmpdir = tmp_path_factory.mktemp("certs")
     ca = trustme.CA()
     # IP address in Subject Alternative Name
@@ -212,7 +222,7 @@ def ipv4_san_proxy_with_server(
 @pytest.fixture
 def ipv6_san_proxy_with_server(
     tmp_path_factory: pytest.TempPathFactory,
-) -> typing.Generator[tuple[ServerConfig, ServerConfig], None, None]:
+) -> typing.Generator[tuple[ServerConfig, ServerConfig]]:
     tmpdir = tmp_path_factory.mktemp("certs")
     ca = trustme.CA()
     # IP addresses in Subject Alternative Name
@@ -229,7 +239,7 @@ def ipv6_san_proxy_with_server(
 @pytest.fixture
 def ipv4_san_server(
     tmp_path_factory: pytest.TempPathFactory,
-) -> typing.Generator[ServerConfig, None, None]:
+) -> typing.Generator[ServerConfig]:
     tmpdir = tmp_path_factory.mktemp("certs")
     ca = trustme.CA()
     # IP address in Subject Alternative Name
@@ -242,7 +252,7 @@ def ipv4_san_server(
 @pytest.fixture
 def ipv6_san_server(
     tmp_path_factory: pytest.TempPathFactory,
-) -> typing.Generator[ServerConfig, None, None]:
+) -> typing.Generator[ServerConfig]:
     if not HAS_IPV6:
         pytest.skip("Only runs on IPv6 systems")
 
@@ -258,7 +268,7 @@ def ipv6_san_server(
 @pytest.fixture
 def ipv6_no_san_server(
     tmp_path_factory: pytest.TempPathFactory,
-) -> typing.Generator[ServerConfig, None, None]:
+) -> typing.Generator[ServerConfig]:
     if not HAS_IPV6:
         pytest.skip("Only runs on IPv6 systems")
 
@@ -272,7 +282,7 @@ def ipv6_no_san_server(
 
 
 @pytest.fixture
-def stub_timezone(request: pytest.FixtureRequest) -> typing.Generator[None, None, None]:
+def stub_timezone(request: pytest.FixtureRequest) -> typing.Generator[None]:
     """
     A pytest fixture that runs the test with a stub timezone.
     """
@@ -288,8 +298,8 @@ def supported_tls_versions() -> typing.AbstractSet[str | None]:
     # disables TLSv1 and TLSv1.1.
     tls_versions = set()
 
-    _server = HTTPSDummyServerTestCase()
-    _server._start_server()
+    _server = HTTPSHypercornDummyServerTestCase
+    _server.setup_class()
     for _ssl_version_name, min_max_version in (
         ("PROTOCOL_TLSv1", ssl.TLSVersion.TLSv1),
         ("PROTOCOL_TLSv1_1", ssl.TLSVersion.TLSv1_1),
@@ -314,7 +324,7 @@ def supported_tls_versions() -> typing.AbstractSet[str | None]:
         else:
             tls_versions.add(_sock.version())
         _sock.close()
-    _server._stop_server()
+    _server.teardown_class()
     return tls_versions
 
 
@@ -347,3 +357,42 @@ def requires_tlsv1_3(supported_tls_versions: typing.AbstractSet[str]) -> None:
         or "TLSv1.3" not in supported_tls_versions
     ):
         pytest.skip("Test requires TLSv1.3")
+
+
+class ErroringHTTPConnection:
+    def __init__(self, *args: typing.Any, **kwargs: typing.Any):
+        raise ValueError(
+            "HTTP/2 support currently only applies to HTTPS, don't use http_version for HTTP tests"
+        )
+
+
+@pytest.fixture(params=["h11", "h2"])
+def http_version(request: pytest.FixtureRequest) -> typing.Generator[str]:
+    orig_HTTPConnection: typing.Any = None
+
+    if request.param == "h2":
+        urllib3.http2.inject_into_urllib3()
+
+        from urllib3 import connection as urllib3_connection
+        from urllib3.connectionpool import HTTPConnectionPool
+
+        orig_HTTPConnection = urllib3_connection.HTTPConnection
+        urllib3_connection.HTTPConnection = ErroringHTTPConnection  # type: ignore[misc,assignment]
+        HTTPConnectionPool.ConnectionCls = ErroringHTTPConnection  # type: ignore[assignment]
+    try:
+        yield request.param
+    finally:
+        if request.param == "h2":
+            urllib3_connection.HTTPConnection = orig_HTTPConnection  # type: ignore[misc]
+            HTTPConnectionPool.ConnectionCls = orig_HTTPConnection
+
+            urllib3.http2.extract_from_urllib3()
+
+
+@pytest.fixture(autouse=True, scope="function")
+def reset_http2_probe_cache() -> typing.Generator[None]:
+    # Always reset the HTTP/2 probe cache per test case.
+    try:
+        yield
+    finally:
+        http2_probe._reset()
